<?php
class Members_Action extends Typecho_Widget implements Widget_Interface_Do
{
    /**
     * @var Widget_Security
     */
    private $security;

    /**
     * 构造函数
     */
    public function __construct($request, $response, $params = NULL)
    {
        parent::__construct($request, $response, $params);
        $this->security = Widget_Security::alloc();
    }

    public function outputStyle()
    {
        header('Content-Type: text/css');
        
        $css = '.members-only {
            max-width: 800px;
            margin: 40px auto;
            background: #fff;
            border-radius: 16px;
            box-shadow: 0 4px 24px rgba(0, 0, 0, 0.06);
            overflow: hidden;
            text-align: center;
            border: 1px solid #e5e7eb;
        }

        .lock-content {
            padding: 48px 32px;
            display: flex;
            flex-direction: column;
            align-items: center;
            gap: 24px;
            position: relative;
        }

        .lock-content::before {
            content: "";
            position: absolute;
            top: 0;
            left: 0;
            right: 0;
            height: 4px;
            background: linear-gradient(90deg, rgb(220, 35, 15), rgb(240, 80, 60));
        }

        .lock-icon {
            width: 64px;
            height: 64px;
            background: #fff1f0;
            border-radius: 50%;
            display: flex;
            align-items: center;
            justify-content: center;
            color: rgb(220, 35, 15);
            margin-bottom: 8px;
        }

        .lock-icon svg {
            width: 28px;
            height: 28px;
        }

        .lock-info {
            max-width: 480px;
        }

        .lock-text {
            font-size: 24px;
            font-weight: 600;
            color: #1f2937;
            margin-bottom: 12px;
            letter-spacing: -0.02em;
        }

        .lock-desc {
            font-size: 16px;
            color: #6b7280;
            line-height: 1.6;
            margin-bottom: 32px;
        }

        .lock-action {
            display: flex;
            flex-direction: column;
            align-items: center;
            gap: 16px;
            width: 100%;
            max-width: 320px;
        }

        .lock-action a {
            display: inline-flex;
            align-items: center;
            justify-content: center;
            width: 100%;
            padding: 14px 28px;
            background: linear-gradient(90deg, rgb(220, 35, 15), rgb(240, 80, 60));
            color: #fff;
            font-size: 16px;
            font-weight: 500;
            border-radius: 12px;
            text-decoration: none;
            transition: all 0.3s ease;
            box-shadow: 0 2px 8px rgba(220, 35, 15, 0.25);
        }

        .lock-action a:hover {
            transform: translateY(-2px);
            box-shadow: 0 4px 12px rgba(220, 35, 15, 0.35);
        }

        /* 深色模式 */
        @media (prefers-color-scheme: dark) {
            .members-only {
                background: #1f2937;
                border-color: #374151;
                box-shadow: 0 4px 24px rgba(0, 0, 0, 0.2);
            }

            .lock-icon {
                background: #2d1715;
                color: rgb(240, 80, 60);
            }

            .lock-text {
                color: #f3f4f6;
            }

            .lock-desc {
                color: #9ca3af;
            }

            .lock-action a {
                background: linear-gradient(90deg, rgb(200, 30, 10), rgb(220, 35, 15));
            }
        }

        /* 响应式布局 */
        @media (max-width: 640px) {
            .members-only {
                margin: 20px 16px;
                border-radius: 12px;
            }

            .lock-content {
                padding: 32px 20px;
            }

            .lock-icon {
                width: 56px;
                height: 56px;
            }

            .lock-text {
                font-size: 20px;
            }

            .lock-desc {
                font-size: 15px;
                margin-bottom: 24px;
            }
        }';
        
        echo $css;
        exit;
    }

    public function editMember()
    {
        // 检查权限
        $this->widget('Widget_User')->pass('administrator');
        
        // 检查安全性
        $this->security->protect();
        
        // 获取表单数据
        $uid = $this->request->get('uid');
        $expire_time = $this->request->get('expire_time');
        $start_time = $this->request->get('start_time');
        
        // 验证数据
        if (empty($uid)) {
            $this->widget('Widget_Notice')->set(_t('请选择用户'), 'error');
            $this->response->goBack();
            return;
        }
        
        if (empty($expire_time)) {
            $this->widget('Widget_Notice')->set(_t('请设置到期时间'), 'error');
            $this->response->goBack();
            return;
        }
        
        try {
            Members_Plugin::saveMember($uid, $expire_time, $start_time);
            
            $db = Typecho_Db::get();
            $user = $db->fetchRow($db->select('name')->from('table.users')->where('uid = ?', $uid));
            $this->widget('Widget_Notice')->set(
                _t('用户 %s 的会员信息已更新', $user['name']), 
                'success'
            );
            
            // 修正重定向地址
            $this->response->redirect(Typecho_Common::url('/admin/extending.php?panel=Members/Admin.php', $this->options->siteUrl));
        } catch (Exception $e) {
            $this->widget('Widget_Notice')->set(_t('保存失败：%s', $e->getMessage()), 'error');
            $this->response->goBack();
        }
    }

    public function deleteMember()
    {
        // 检查权限
        $this->widget('Widget_User')->pass('administrator');
        
        // 检查安全性
        $this->security->protect();
        
        $uid = $this->request->get('uid');
        
        if (empty($uid)) {
            $this->widget('Widget_Notice')->set(_t('参数错误'), 'error');
            $this->response->goBack();
            return;
        }
        
        try {
            $db = Typecho_Db::get();
            $user = $db->fetchRow($db->select('name')->from('table.users')->where('uid = ?', $uid));
            $db->query($db->delete('table.members')->where('uid = ?', $uid));
            
            $this->widget('Widget_Notice')->set(
                _t('已删除用户 %s 的会员信息', $user['name']), 
                'success'
            );
            
            // 修正重定向地址
            $this->response->redirect(Typecho_Common::url('/admin/extending.php?panel=Members/Admin.php', $this->options->siteUrl));
        } catch (Exception $e) {
            $this->widget('Widget_Notice')->set(_t('删除失败：%s', $e->getMessage()), 'error');
            $this->response->goBack();
        }
    }

    public function action()
    {
        $this->on($this->request->is('do=outputStyle'))->outputStyle();
        $this->on($this->request->is('do=editMember'))->editMember();
        $this->on($this->request->is('do=deleteMember'))->deleteMember();
    }
} 