<?php
/**
 * 记忆流会员插件
 * 
 * @package Members
 * @author <a href="https://wpa.qq.com/msgrd?v=3&uin=3295403478&site=qq&menu=yes" target="_blank">记忆流</a>
 * @version 1.0.0
 * @link https://f1owmem.com
 * 转载请注明出处
 */
class Members_Plugin implements Typecho_Plugin_Interface
{
/**
 * 激活插件
 */
public static function activate()
{
    $db = Typecho_Db::get();
    $prefix = $db->getPrefix();
    
    try {
        // 检查表是否存在
            $tableExists = false;
            $result = $db->fetchAll($db->query("SHOW TABLES LIKE '{$prefix}members'"));
            $tableExists = count($result) > 0;
            
            // 如果表不存在，创建新表
            if (!$tableExists) {
                $sql = "CREATE TABLE IF NOT EXISTS `" . $prefix . "members` (
                    `uid` int(11) unsigned NOT NULL,
                    `start_time` int(11) NOT NULL DEFAULT 0,
                    `expire_time` int(11) DEFAULT NULL,
                    PRIMARY KEY (`uid`),
                    CONSTRAINT `members_uid_fk` FOREIGN KEY (`uid`) REFERENCES `{$prefix}users` (`uid`) ON DELETE CASCADE ON UPDATE CASCADE
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";
                
                $db->query($sql);
            }
            
            // 添加路由和过滤器
            Helper::addPanel(1, 'Members/Admin.php', '会员管理', '管理会员信息', 'administrator');
            Helper::addAction('members-edit', 'Members_Action');
            Helper::addAction('members-style', 'Members_Action');
            Helper::addRoute('members-css', '/members/style.css', 'Members_Action', 'outputStyle');
            
            // 添加内容过滤器
            Typecho_Plugin::factory('Widget_Abstract_Contents')->contentEx = array('Members_Plugin', 'contentFilter');
            Typecho_Plugin::factory('Widget_Abstract_Contents')->excerptEx = array('Members_Plugin', 'contentFilter');
            
            return _t('插件启用成功');
        } catch (Typecho_Db_Exception $e) {
            return _t('插件启用失败：%s', $e->getMessage());
        }
    }
    
    /**
     * 禁用插件
     */
    public static function deactivate()
    {
        // 移除路由和面板
        Helper::removeRoute('members-css');
        Helper::removeAction('members-edit');  // 移除编辑会员的 action
        Helper::removeAction('members-style');
        Helper::removePanel(1, 'Members/Admin.php');
        
        // 检查是否需要删除数据
        $options = Helper::options()->plugin('Members');
        if (!$options->keepData) {
            try {
                $db = Typecho_Db::get();
                $prefix = $db->getPrefix();
                $db->query("DROP TABLE IF EXISTS `" . $prefix . "members`");
            } catch (Exception $e) {
                return _t('插件禁用成功，但删除数据表失败：%s', $e->getMessage());
            }
        }
        
        return _t('插件已禁用' . ($options->keepData ? '，数据已保留' : '，数据已删除'));
    }
    
    /**
     * 获取插件配置面板
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        // 添加数据保留选项
        $keepData = new Typecho_Widget_Helper_Form_Element_Radio(
            'keepData',
            array(
                0 => _t('禁用时删除数据'),
                1 => _t('禁用时保留数据')
            ),
            0,
            _t('数据保留设置'),
            _t('禁用插件时是否保留数据库中的会员信息')
        );
        $form->addInput($keepData);
        
        // 锁定提示文字
        $lockText = new Typecho_Widget_Helper_Form_Element_Text(
            'lockText', 
            null,
            '本篇内容为付费限定',
            _t('锁定提示文字'),
            _t('显示在锁定内容上方的主要提示文字')
        );
        $form->addInput($lockText);
        
        // 描述文字
        $lockDesc = new Typecho_Widget_Helper_Form_Element_Text(
            'lockDesc', 
            null,
            '若想获得观看权限，请购买此文章所属方案',
            _t('描述文字'),
            _t('显示在主要提示文字下方的详细说明')
        );
        $form->addInput($lockDesc);
        
        // 购买提示信息
        $buyMessage = new Typecho_Widget_Helper_Form_Element_Textarea(
            'buyMessage', 
            null,
            '需要成为VIP请联系 <a href="https://wpa.qq.com/msgrd?v=3&uin=3295403478&site=qq&menu=yes" target="_blank">QQ: 3295403478</a> 购买',
            _t('购买提示信息'),
            _t('支持HTML代码，可以添加联系方式和购买说明')
        );
        $form->addInput($buyMessage);
    }

    /**
     * 个人用户的配置面板
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){}
    
    /**
     * 保存会员信息
     */
    public static function saveMember($uid, $expire_time, $start_time = null)
    {
        $db = Typecho_Db::get();
        
        // 确保时间戳是当天23:59:59
        $expire_timestamp = strtotime($expire_time . ' 23:59:59');
        $start_timestamp = $start_time ? strtotime($start_time . ' 00:00:00') : time();
        
        try {
            // 检查是否已存在
            $member = $db->fetchRow($db->select()->from('table.members')->where('uid = ?', $uid));
            
            if ($member) {
                // 更新
                return $db->query($db->update('table.members')
                    ->rows([
                        'start_time' => $start_timestamp,
                        'expire_time' => $expire_timestamp
                    ])
                    ->where('uid = ?', $uid));
            } else {
                // 新增
                return $db->query($db->insert('table.members')
                    ->rows([
                        'uid' => $uid,
                        'start_time' => $start_timestamp,
                        'expire_time' => $expire_timestamp
                    ]));
            }
        } catch (Exception $e) {
            throw new Exception(_t('保存失败：%s', $e->getMessage()));
        }
    }
    
    /**
     * 获取所有会员
     */
    public static function getAllMembers()
    {
        $db = Typecho_Db::get();
        try {
            return $db->fetchAll($db->select('m.*, u.name')
                ->from('table.members m')
                ->join('table.users u', 'm.uid = u.uid')
                ->order('m.expire_time', Typecho_Db::SORT_DESC));
        } catch (Exception $e) {
            throw new Exception(_t('获取会员列表失败：%s', $e->getMessage()));
        }
    }
    
    /**
     * 内容过滤
     */
    public static function contentFilter($content, $widget = null, $lastResult = null)
    {
        // 如果已经有处理结果，直接返回
        if ($lastResult) {
            return $lastResult;
        }

        // 支持多种短代码格式
        $patterns = [
            '/\[vip\](.*?)\[\/vip\]/is',
            '/\[member\](.*?)\[\/member\]/is',
            '/\{vip\}(.*?)\{\/vip\}/is'
        ];
        
        // 检查是否包含会员标记
        $hasVipContent = false;
        foreach ($patterns as $pattern) {
            if (preg_match($pattern, $content)) {
                $hasVipContent = true;
                break;
            }
        }
        
        if (!$hasVipContent) {
            return $content;
        }

        // 检查用户权限
        $canAccess = false;
        $user = Typecho_Widget::widget('Widget_User');
        $currentTime = time();
        
        if ($user->hasLogin()) {
            // 管理员直接允许访问
            if ($user->group == 'administrator') {
                $canAccess = true;
            } else {
                try {
                    $db = Typecho_Db::get();
                    
                    // 检查会员状态
                    $member = $db->fetchRow($db->select()
                        ->from('table.members')
                        ->where('uid = ?', $user->uid)
                        ->where('start_time <= ?', $currentTime)
                        ->where('expire_time > ?', $currentTime));
                    
                    if ($member) {
                        $canAccess = true;
                    }
                } catch (Exception $e) {
                    error_log("Member check error: " . $e->getMessage());
                }
            }
        }

        // 检查是否已经添加了样式
        static $styleAdded = false;
        if (!$styleAdded && !$canAccess) {
            $options = Helper::options();
            $styleUrl = Typecho_Common::url('/action/members-style?do=outputStyle', $options->index);
            echo '<link rel="stylesheet" href="' . $styleUrl . '">';
            $styleAdded = true;
        }

        // 获取插件配置
        $pluginOptions = Helper::options()->plugin('Members');
        $lockText = empty($pluginOptions->lockText) ? '此内容仅限会员查看' : $pluginOptions->lockText;
        $lockDesc = empty($pluginOptions->lockDesc) ? '开通会员即可查看完整内容' : $pluginOptions->lockDesc;
        $buyMessage = empty($pluginOptions->buyMessage) ? '请联系管理员开通会员' : $pluginOptions->buyMessage;
        
        // 生成登录链接
        $options = Helper::options();
        $loginUrl = defined('__TYPECHO_ADMIN_DIR__') ? 
            Typecho_Common::url('login.php', $options->adminUrl) : 
            Typecho_Common::url('admin/login.php', $options->siteUrl);
        
        // 非VIP内容模板
        $nonVipTemplate = '<div class="members-only">
            <div class="lock-content">
                <div class="lock-icon">
                    <svg viewBox="0 0 24 24" width="24" height="24">
                        <path d="M18 8h-1V6c0-2.76-2.24-5-5-5S7 3.24 7 6v2H6c-1.1 0-2 .9-2 2v10c0 1.1.9 2 2 2h12c1.1 0 2-.9 2-2V10c0-1.1-.9-2-2-2zM9 6c0-1.66 1.34-3 3-3s3 1.34 3 3v2H9V6zm9 14H6V10h12v10zm-6-3c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2 .9 2 2 2z" fill="currentColor"/>
                    </svg>
                </div>
                <div class="lock-info">
                    <div class="lock-text">' . $lockText . '</div>
                    <div class="lock-desc">' . $lockDesc . '</div>
                </div>
                <div class="lock-action">' . 
                    ($user->hasLogin() ? $buyMessage : 
                    '<a href="' . $loginUrl . '" rel="nofollow">' . _t('登录后查看') . '</a>') . 
                '</div>
            </div>
        </div>';
        
        // 根据权限替换内容
        if ($canAccess) {
            foreach ($patterns as $pattern) {
                $content = preg_replace($pattern, '$1', $content);
            }
        } else {
            foreach ($patterns as $pattern) {
                $content = preg_replace($pattern, $nonVipTemplate, $content);
            }
        }
        
        return $content;
    }

    /**
     * 输出样式
     */
    public static function outputStyle()
    {
        header('Content-Type: text/css');
        
        $css = '.members-only {
            margin: 20px 0;
            background: linear-gradient(to right, #f8fafc, #f1f5f9);
            border: 1px solid #e2e8f0;
            border-radius: 12px;
            padding: 2px;
            position: relative;
            overflow: hidden;
        }

        .members-only::before {
            content: "";
            position: absolute;
            inset: 0;
            background: linear-gradient(90deg, #60a5fa20, #3b82f620);
            mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            -webkit-mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            mask-composite: exclude;
            -webkit-mask-composite: xor;
            padding: 1px;
            border-radius: 12px;
        }

        .lock-content {
            display: flex;
            align-items: center;
            gap: 16px;
            padding: 20px;
            background: #fff;
            border-radius: 10px;
            position: relative;
            z-index: 1;
        }

        .lock-icon {
            display: flex;
            align-items: center;
            justify-content: center;
            width: 42px;
            height: 42px;
            background: linear-gradient(135deg, #f8fafc, #f1f5f9);
            border: 1px solid #e2e8f0;
            border-radius: 10px;
            color: #64748b;
            flex-shrink: 0;
            position: relative;
        }

        .lock-icon::after {
            content: "";
            position: absolute;
            inset: -2px;
            border-radius: 11px;
            padding: 1px;
            background: linear-gradient(135deg, #60a5fa40, #3b82f440);
            mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            -webkit-mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            mask-composite: exclude;
            -webkit-mask-composite: xor;
        }

        .lock-icon svg {
            width: 22px;
            height: 22px;
        }

        .lock-info {
            flex-grow: 1;
            background: linear-gradient(135deg, #f8fafc, #f1f5f9);
            border: 1px solid #e2e8f0;
            border-radius: 10px;
            padding: 14px 18px;
            position: relative;
        }

        .lock-info::after {
            content: "";
            position: absolute;
            inset: -2px;
            border-radius: 11px;
            padding: 1px;
            background: linear-gradient(135deg, #60a5fa20, #3b82f420);
            mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            -webkit-mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            mask-composite: exclude;
            -webkit-mask-composite: xor;
        }

        .lock-text {
            font-size: 15px;
            font-weight: 600;
            color: #334155;
            margin: 0;
        }

        .lock-desc {
            font-size: 14px;
            color: #64748b;
            margin: 4px 0 0 0;
        }

        .lock-action {
            flex-shrink: 0;
            background: linear-gradient(135deg, #f8fafc, #f1f5f9);
            border: 1px solid #e2e8f0;
            border-radius: 10px;
            padding: 14px 18px;
            position: relative;
        }

        .lock-action::after {
            content: "";
            position: absolute;
            inset: -2px;
            border-radius: 11px;
            padding: 1px;
            background: linear-gradient(135deg, #60a5fa20, #3b82f420);
            mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            -webkit-mask: linear-gradient(#fff 0 0) content-box, linear-gradient(#fff 0 0);
            mask-composite: exclude;
            -webkit-mask-composite: xor;
        }

        .lock-action a {
            color: #3b82f6;
            text-decoration: none;
            font-size: 14px;
            font-weight: 500;
            white-space: nowrap;
            transition: all 0.3s ease;
        }

        .lock-action a:hover {
            color: #2563eb;
        }

        /* 深色模式 */
        @media (prefers-color-scheme: dark) {
            .members-only {
                background: linear-gradient(to right, #0f172a, #1e293b);
                border-color: #334155;
            }
            
            .lock-content {
                background: #0f172a;
            }
            
            .lock-icon,
            .lock-info,
            .lock-action {
                background: linear-gradient(135deg, #1e293b, #0f172a);
                border-color: #334155;
            }
            
            .lock-text {
                color: #e2e8f0;
            }
            
            .lock-desc {
                color: #94a3b8;
            }
            
            .lock-action a {
                color: #60a5fa;
            }
            
            .lock-action a:hover {
                color: #3b82f6;
            }
        }

        /* 响应式布局 */
        @media (max-width: 640px) {
            .lock-content {
                flex-direction: column;
                align-items: stretch;
                gap: 12px;
            }
            
            .lock-action {
                text-align: center;
            }
        }';
        
        echo $css;
        exit;
    }
} 